@@ -1,4 +1,4 @@
-#' Orthogonalized Go/Nogo Task (Cavanagh et al., 2013)
+#' Orthogonalized Go/Nogo Task 
 #' 
 #' @description 
 #' Hierarchical Bayesian Modeling of the Orthogonolized Go/Nogo Task using the following parameters: "xi" (noise), "ep" (learning rate), "b" (action bias), "pi" (Pavlovian bias), "rhoRew" (reward sensitivity), and "rhoPun" (punishment sensitivity)
@@ -16,17 +16,20 @@
 #' @param indPars Character value specifying how to summarize individual parameters. Current options are: "mean", "median", or "mode".
 #' @param saveDir Path to directory where .RData file of model output (\code{modelData}) can be saved. Leave blank if not interested.
 #' @param email Character value containing email address to send notification of completion. Leave blank if not interested. 
+#' @param modelRegressor Exporting model-based regressors (Q(Go), Q(NoGo))? TRUE or FALSE. 
+#' @param adapt_delta Floating point number representing the target acceptance probability of a new sample in the MCMC chain. Must be between 0 and 1. See \bold{Details} below.
+#' @param stepsize Integer value specifying the size of each leapfrog step that the MCMC sampler can take on each new iteration. See \bold{Details} below.
+#' @param max_treedepth Integer value specifying how many leapfrog steps that the MCMC sampler can take on each new iteration. See \bold{Details} below. 
 #' 
-#' @return \code{modelData}  A class \code{'hBayesDM'} object with the following components:
+#' @return \code{modelData}  A class \code{"hBayesDM"} object with the following components:
 #' \describe{
-#'  \item{\code{model}}{Character string with the name of the model ("gng_m4").}
-#'  \item{\code{allIndPars}}{\code{'data.frame'} containing the summarized parameter 
-#'    values (as specified by \code{'indPars'}) for each subject.}
-#'  \item{\code{parVals}}{A \code{'list'} where each element contains posterior samples
-#'    over different model parameters. }
-#'  \item{\code{fit}}{A class \code{'stanfit'} object containing the fitted model.}
-#' }
-#' 
+#'  \item{\code{model}}{Character string with the name of the model (\code{"gng_m4"}).}
+#'  \item{\code{allIndPars}}{\code{"data.frame"} containing the summarized parameter values (as specified by \code{"indPars"}) for each subject.}
+#'  \item{\code{parVals}}{A \code{"list"} where each element contains posterior samples over different model parameters. }
+#'  \item{\code{fit}}{A class \code{"stanfit"} object containing the fitted model.}
+#'  \item{\code{rawdata}}{\code{"data.frame"} containing the raw data used to fit the model, as specified by the user.}
+#' }  
+#'
 #' @importFrom rstan stan rstan_options extract
 #' @importFrom modeest mlv
 #' @importFrom mail sendmail
@@ -51,25 +54,41 @@
 #' \strong{*}Note: The data.txt file may contain other columns of data (e.g. "Reaction_Time", "trial_number", etc.), but only the data with the column
 #' names listed above will be used for analysis/modeling. As long as the columns above are present and labelled correctly,
 #' there is no need to remove other miscellaneous data columns.    
-#'  
+#' 
 #' \strong{nwarmup} is a numerical value that specifies how many MCMC samples should not be stored upon the 
 #' beginning of each chain. For those familiar with Bayesian methods, this value is equivalent to a burn-in sample. 
 #' Due to the nature of MCMC sampling, initial values (where the sampling chain begins) can have a heavy influence 
-#' on the generated posterior distributions. The \strong{nwarmup} argument can be set to a high number in order to curb the 
+#' on the generated posterior distributions. The \code{nwarmup} argument can be set to a high number in order to curb the 
 #' effects that initial values have on the resulting posteriors.  
 #' 
 #' \strong{nchain} is a numerical value that specifies how many chains (i.e. independent sampling sequences) should be
 #' used to draw samples from the posterior distribution. Since the posteriors are generated from a sampling 
 #' process, it is good practice to run multiple chains to ensure that a representative posterior is attained. When
 #' sampling is completed, the multiple chains may be checked for convergence with the \code{plot(myModel, type = "trace")}
-#' command. The chains should resemble a "furry caterpillar". 
+#' command. The chains should resemble a "furry caterpillar".
 #' 
 #' \strong{nthin} is a numerical value that specifies the "skipping" behavior of the MCMC samples being chosen 
-#' to generate the posterior distributions. By default, \strong{nthin} is equal to 1, hence every sample is used to 
+#' to generate the posterior distributions. By default, \code{nthin} is equal to 1, hence every sample is used to 
 #' generate the posterior. 
 #' 
+#' \strong{Contol Parameters:} adapt_delta, stepsize, and max_treedepth are advanced options that give the user more control 
+#' over Stan's MCMC sampler. The Stan creators recommend that only advanced users change the default values, as alterations
+#' can profoundly change the sampler's behavior. Refer to Hoffman & Gelman (2014, Journal of Machine Learning Research) for 
+#' more information on the functioning of the sampler control parameters. One can also refer to section 58.2 of the  
+#' \href{http://mc-stan.org/documentation/}{Stan User's Manual} for a less technical description of these arguments. 
+#' 
 #' @export 
 #' 
+#' @references 
+#' Cavanagh, J. F., Eisenberg, I., Guitart-Masip, M., Huys, Q., & Frank, M. J. (2013). Frontal Theta Overrides Pavlovian 
+#' Learning Biases. Journal of Neuroscience, 33(19), 8541-8548. http://doi.org/10.1523/JNEUROSCI.5754-12.2013
+#' 
+#' Hoffman, M. D., & Gelman, A. (2014). The No-U-turn sampler: adaptively setting path lengths in Hamiltonian Monte Carlo. The 
+#' Journal of Machine Learning Research, 15(1), 1593-1623.
+#' 
+#' @seealso 
+#' We refer users to our in-depth tutorial for an example of using hBayesDM: \url{https://rpubs.com/CCSL/hBayesDM}
+#' 
 #' @examples 
 #' \dontrun{
 #' # Run the model and store results in "output"
@@ -82,19 +101,30 @@
 #' printFit(output)
 #' }
 
-gng_m4 <- function(data     = NULL,
-                   niter    = 5000, 
-                   nwarmup  = 2000, 
-                   nchain   = 1,
-                   ncore    = 1, 
-                   nthin    = 1,
-                   inits    = "fixed",  
-                   indPars  = "mean", 
-                   saveDir  = NULL,
-                   email    = NULL) {
+gng_m4 <- function(data          = "choose",
+                   niter         = 5000, 
+                   nwarmup       = 2000, 
+                   nchain        = 1,
+                   ncore         = 1, 
+                   nthin         = 1,
+                   inits         = "fixed",  
+                   indPars       = "mean", 
+                   saveDir       = NULL,
+                   email         = NULL,
+                   modelRegressor= FALSE,
+                   adapt_delta   = 0.8,
+                   stepsize      = 1,
+                   max_treedepth = 10 ) {
   
   # Path to .stan model file
-  modelPath <- system.file("stan", "gng_m4.stan", package="hBayesDM")
+  if (modelRegressor) { # model regressors (for model-based neuroimaging, etc.)
+    modelPath <- system.file("stan", "gng_m4_reg.stan", package="hBayesDM")
+    cat("************************************\n")
+    cat("** Extract model-based regressors **\n")
+    cat("************************************\n")
+  } else {
+    modelPath <- system.file("stan", "gng_m4.stan", package="hBayesDM")
+  }
   
   # To see how long computations take
   startTime <- Sys.time()    
@@ -102,6 +132,8 @@ gng_m4 <- function(data     = NULL,
   # For using example data
   if (data=="example") {
     data <- system.file("extdata", "gng_exampleData.txt", package = "hBayesDM")
+  } else if (data=="choose") {
+    data <- file.choose()
   }
 
   # Load data
@@ -122,6 +154,10 @@ gng_m4 <- function(data     = NULL,
                "xi", "ep", "b", "pi", "rhoRew", "rhoPun",
                "log_lik")
   
+  if (modelRegressor) { # model regressors (for model-based neuroimaging, etc.)
+    POI = c(POI, "Qgo", "Qnogo", "Wgo", "Wnogo", "SV")
+  }
+  
   modelName <- "gng_m4"
   
   # Information for user
@@ -186,14 +222,14 @@ gng_m4 <- function(data     = NULL,
     }  
     genInitList <- function() {
       list(
-        mu_p     = c(qnorm(inits_fixed[1]), qnorm(inits_fixed[2]), inits_fixed[3], inits_fixed[4], log(inits_fixed[5]), log(inits_fixed[6]) ),
-        sigma    = c(1, 1, 1, 1, 1, 1),
-        xi_p     = rep(qnorm(inits_fixed[1]), numSubjs),
-        ep_p     = rep(qnorm(inits_fixed[2]), numSubjs),
-        b_p      = rep(inits_fixed[3], numSubjs),
-        pi_p     = rep(inits_fixed[4], numSubjs),
-        rhoRew_p = rep(log(inits_fixed[5]), numSubjs),
-        rhoPun_p = rep(log(inits_fixed[6]), numSubjs)
+        mu_p      = c(qnorm(inits_fixed[1]), qnorm(inits_fixed[2]), inits_fixed[3], inits_fixed[4], log(inits_fixed[5]), log(inits_fixed[6]) ),
+        sigma     = c(1.0, 1.0, 1.0, 1.0, 1.0, 1.0),
+        xi_pr     = rep(qnorm(inits_fixed[1]), numSubjs),
+        ep_pr     = rep(qnorm(inits_fixed[2]), numSubjs),
+        b_pr      = rep(inits_fixed[3], numSubjs),
+        pi_pr     = rep(inits_fixed[4], numSubjs),
+        rhoRew_pr = rep(log(inits_fixed[5]), numSubjs),
+        rhoPun_pr = rep(log(inits_fixed[6]), numSubjs)
       )
     }
   } else {
@@ -225,8 +261,12 @@ gng_m4 <- function(data     = NULL,
                      init   = genInitList, 
                      iter   = niter, 
                      chains = nchain,
-                     thin   = nthin)
+                     thin   = nthin,
+                     control = list(adapt_delta   = adapt_delta, 
+                                    max_treedepth = max_treedepth, 
+                                    stepsize      = stepsize) )
   
+  ## Extract parameters
   parVals <- rstan::extract(fit, permuted=T)
   
   xi     <- parVals$xi
@@ -236,8 +276,10 @@ gng_m4 <- function(data     = NULL,
   rhoRew <- parVals$rhoRew
   rhoPun <- parVals$rhoPun
   
+  # Individual parameters (e.g., individual posterior means)
   allIndPars <- array(NA, c(numSubjs, numPars))
   allIndPars <- as.data.frame(allIndPars)
+  
   for (i in 1:numSubjs) {
     if (indPars=="mean") {
       allIndPars[i, ] <- c( mean(xi[, i]), 
@@ -272,9 +314,43 @@ gng_m4 <- function(data     = NULL,
                             "rhoPun", 
                             "subjID")
 
-  # Wrap up data into a list
-  modelData        <- list(modelName, allIndPars, parVals, fit)
-  names(modelData) <- c("model", "allIndPars", "parVals", "fit")
+  # model-based regressors?
+  if (modelRegressor) {
+    if (indPars=="mean") {
+      Qgo   = apply(parVals$Qgo, c(2,3), mean)
+      Qnogo = apply(parVals$Qnogo, c(2,3), mean)
+      Wgo   = apply(parVals$Wgo, c(2,3), mean)
+      Wnogo = apply(parVals$Wnogo, c(2,3), mean)
+      SV    = apply(parVals$SV, c(2,3), mean)
+    } else if (indPars=="median") {
+      Qgo   = apply(parVals$Qgo, c(2,3), median)
+      Qnogo = apply(parVals$Qnogo, c(2,3), median)
+      Wgo   = apply(parVals$Wgo, c(2,3), median)
+      Wnogo = apply(parVals$Wnogo, c(2,3), median)
+      SV    = apply(parVals$SV, c(2,3), median)
+    } else if (indPars=="mode") {
+      Qgo   = apply(parVals$Qgo, c(2,3), modeest::mfv)   # using mfv function
+      Qnogo = apply(parVals$Qnogo, c(2,3), modeest::mfv) # using mfv function
+      Wgo   = apply(parVals$Wgo, c(2,3), modeest::mfv)   # using mfv function
+      Wnogo = apply(parVals$Wnogo, c(2,3), modeest::mfv) # using mfv function
+      SV    = apply(parVals$SV, c(2,3), modeest::mfv)    # using mfv function
+    }
+    # initialize modelRegressor and add model-based regressors
+    modelRegressor = NULL
+    modelRegressor$Qgo = Qgo
+    modelRegressor$Qnogo = Qnogo
+    modelRegressor$Wgo = Wgo
+    modelRegressor$Wnogo = Wnogo
+    modelRegressor$SV = SV
+    
+    # Wrap up data into a list
+    modelData        <- list(modelName, allIndPars, parVals, fit, rawdata, modelRegressor)
+    names(modelData) <- c("model", "allIndPars", "parVals", "fit", "rawdata", "modelRegressor")
+  } else {
+    # Wrap up data into a list
+    modelData        <- list(modelName, allIndPars, parVals, fit, rawdata)
+    names(modelData) <- c("model", "allIndPars", "parVals", "fit", "rawdata")
+  }
   class(modelData) <- "hBayesDM"
   
   # Total time of computations
@@ -304,7 +380,3 @@ gng_m4 <- function(data     = NULL,
   
   return(modelData)
 }
-
-
-
-
