@@ -138,8 +138,8 @@ plot_prl_ewa <- function( obj, fontSize = 10, ncols = 3, binSize = 30 ) {
 
 plot_prl_rp <- function( obj, fontSize = 10, ncols = 3, binSize = 30 ) {
   pars = obj$parVals
-  h1 = plotDist(sample = pars$mu_Apun, fontSize = fontSize, binSize = binSize, xLim = c(0,1), xLab = paste(expression('A'['pun']), " (Punishment Learning Rate)"))
-  h2 = plotDist(sample = pars$mu_Arew, fontSize = fontSize, binSize = binSize, xLim = c(0,1), xLab = paste(expression('A'['rew']), " (Reward Learning Rate)"))
+  h1 = plotDist(sample = pars$mu_Apun, fontSize = fontSize, binSize = binSize, xLim = c(0,1), xLab = expression(paste(A[pun], " (Punishment Learning Rate)")))
+  h2 = plotDist(sample = pars$mu_Arew, fontSize = fontSize, binSize = binSize, xLim = c(0,1), xLab = expression(paste(A[rew], " (Reward Learning Rate)")))
   h3 = plotDist(sample = pars$mu_beta, fontSize = fontSize, binSize = binSize, xLab = expression(paste(beta, " (Inverse Temp.)")))
   h_all = multiplot(h1, h2, h3, cols = ncols)
   return(h_all)
@@ -188,3 +188,4 @@ plot_ug_delta <- function( obj, fontSize = 10, ncols = 3, binSize = 30 ) {
   return(h_all)
 }
 
+
